/*
 * ExtractPanel.java
 *
 * Created on October 28, 2007, 3:16 PM
 */

package com.meapsoft.newgui;

import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import util.RTSI;

import com.meapsoft.featextractors.FeatureExtractor;
import com.meapsoft.gui.GUIUtils;

/**
 *
 * @author  Mike
 */
public class ExtractPanel extends BasePanel implements ListSelectionListener
{
	//save this selected feature
    FeatureExtractorCheckBox mSelectedFeature = null;
	
	
    /** Creates new form ExtractPanel */
    public ExtractPanel(MeapGUI mainScreen)
    {
    	//pass this up to our main screen
    	super(mainScreen);
        
    	//init our layout and stuff
        initComponents();
        
        //initialize the checkboxes here
        initCheckboxes();
        
        //initialze the rest of the panel
        initPanel();
    }
    
    ////////////////////////////////////////////////////////////////////////
    // initialization methods
    ////////////////////////////////////////////////////////////////////////
    private void initCheckboxes() 
    {
    	//sniff out the feature extrators first
    	Vector featureExtractors = sniffFeatureExtractors();
    	
    	//iterate and create our checkboxes
        for(int i = 0; i < featureExtractors.size(); i++)
        {
        	//get the name
        	String name = featureExtractors.get(i).toString();
        	
        	//create the checkbox here and add it
        	FeatureExtractorCheckBox chk = new FeatureExtractorCheckBox(name);
            mFeatureExtractorList.add(chk);
        }
	}
    
    private void initPanel()
    {
    	//set this to false
    	mDetailsWrapper.setVisible(false);
    	
    	//add us as a mouse listener of the checkBox
    	mFeatureExtractorList.addListSelectionLister(this);
    	
    	//initialze the error label
    	ImageIcon icon = new ImageIcon(getClass().getResource("/com/exclaim.png"));
    	mWeightErrorLbl.setIcon(icon);
    	mWeightErrorLbl.setText("Invalid weight. It must be between 0 and 1");
    	mWeightErrorLbl.setVisible(false);
    }

    ////////////////////////////////////////////////////////////////////////
    // sniffFeatureExtractors() - looks for feature extractors
    ////////////////////////////////////////////////////////////////////////
	private Vector sniffFeatureExtractors()
	{
        Vector v = null;
        try 
        {
        	//look in feature extractor directory
            v = RTSI.findnames("com.meapsoft.featextractors", Class.forName("com.meapsoft.featextractors.FeatureExtractor"));

            //also look in current directory
            Vector v2 = RTSI.findnames(System.getProperty("user.dir"), Class.forName("com.meapsoft.featextractors.FeatureExtractor")); 
            
            //add these to original vector
            if(v2 != null)
                v.addAll(v2);
            
            //get the extractor itself associated with this 
            for (int i = 0; i < v.size(); i++)
            {
				FeatureExtractor f = null;
				
				try
				{
					String name = "com.meapsoft.featextractors." + (String)v.elementAt(i);
					f = (FeatureExtractor)(Class.forName(name).newInstance());
				}
				catch (Exception e)
				{
					GUIUtils.ShowDialog("", GUIUtils.MESSAGE, mMainScreen);	
                    //ShowDialog(e, "", FATAL_ERROR);
				}
            }
        }
        catch(ClassNotFoundException e)
        {
			GUIUtils.ShowDialog("", GUIUtils.FATAL_ERROR, mMainScreen);	
        }

        //return this vector
        return v;
	}
    
    ////////////////////////////////////////////////////////////////////////
    // valueChanged() - callback for when our value is changed
    ////////////////////////////////////////////////////////////////////////
	public void valueChanged(ListSelectionEvent e) 
	{
		//set the details wrapper to be true here
		mDetailsWrapper.setVisible(true);
		
		//get the selected feature at this point (only one allowed at a time)
		mSelectedFeature = (FeatureExtractorCheckBox)mFeatureExtractorList.getSelectedValue();
		
		//update all the data in the details now
		mFeatureNameLbl.setText(mSelectedFeature.getName());
		mFeatureDescTxt.setText(mSelectedFeature.getDescription());
		mWeightTxt.setText(String.valueOf(mSelectedFeature.getWeight()));
	}
	
    ////////////////////////////////////////////////////////////////////////
    // run() - does all the work for this guy
    ////////////////////////////////////////////////////////////////////////
	public synchronized int run()
    {
    	return 0;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jLabel4 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jTextField1 = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        mFeatureExtractorList = new com.meapsoft.newgui.checkboxlist.CheckBoxList();
        jPanel1 = new javax.swing.JPanel();
        mDetailsWrapper = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        mFeatureNameLbl = new javax.swing.JLabel();
        mWeightTxt = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        mFeatureDescTxt = new javax.swing.JTextPane();
        mWeightErrorLbl = new javax.swing.JLabel();

        jLabel4.setText("The feature extractor will search for specific features in the segments and extract them");

        jLabel1.setText("Step 1: Select one or more features to extract. Click to learn more about a feature");

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(""));
        jCheckBox1.setText("Likelihood");
        jCheckBox1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox1.setMargin(new java.awt.Insets(0, 0, 0, 0));

        jTextField1.setText("1");

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jCheckBox1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 21, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(382, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jCheckBox1)
                    .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(45, Short.MAX_VALUE))
        );

        jLabel5.setText("Step 2: Select Meta-features to extract");

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Details"));
        jLabel2.setForeground(new java.awt.Color(102, 102, 255));
        jLabel2.setText("Feature:");

        jLabel3.setForeground(new java.awt.Color(102, 102, 255));
        jLabel3.setText("Description:");

        jLabel6.setForeground(new java.awt.Color(153, 153, 255));
        jLabel6.setText("Weight:");

        mFeatureNameLbl.setText("jLabel7");

        mWeightTxt.setText("1");
        mWeightTxt.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                mWeightTxtKeyReleased(evt);
            }
        });

        mFeatureDescTxt.setBorder(null);
        mFeatureDescTxt.setEditable(false);
        mFeatureDescTxt.setOpaque(false);
        jScrollPane2.setViewportView(mFeatureDescTxt);

        mWeightErrorLbl.setText("Invalid weight. It must be between 0 and 1");

        org.jdesktop.layout.GroupLayout mDetailsWrapperLayout = new org.jdesktop.layout.GroupLayout(mDetailsWrapper);
        mDetailsWrapper.setLayout(mDetailsWrapperLayout);
        mDetailsWrapperLayout.setHorizontalGroup(
            mDetailsWrapperLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, mDetailsWrapperLayout.createSequentialGroup()
                .add(mDetailsWrapperLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, mDetailsWrapperLayout.createSequentialGroup()
                        .add(40, 40, 40)
                        .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 220, Short.MAX_VALUE))
                    .add(mDetailsWrapperLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(mWeightErrorLbl, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 250, Short.MAX_VALUE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, mDetailsWrapperLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(mDetailsWrapperLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel3)
                            .add(jLabel2)))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, mDetailsWrapperLayout.createSequentialGroup()
                        .add(41, 41, 41)
                        .add(mFeatureNameLbl))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, mDetailsWrapperLayout.createSequentialGroup()
                        .add(37, 37, 37)
                        .add(mWeightTxt, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 65, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, mDetailsWrapperLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel6)))
                .addContainerGap())
        );
        mDetailsWrapperLayout.setVerticalGroup(
            mDetailsWrapperLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(mDetailsWrapperLayout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel2)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mFeatureNameLbl)
                .add(16, 16, 16)
                .add(jLabel3)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 65, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel6)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mWeightTxt, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mWeightErrorLbl))
        );

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(mDetailsWrapper, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(mDetailsWrapper, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 421, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .addContainerGap(101, Short.MAX_VALUE))
            .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 508, Short.MAX_VALUE)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(mFeatureExtractorList, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 153, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(29, 29, 29)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel5)
                .addContainerGap(308, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(11, 11, 11)
                .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(mFeatureExtractorList, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 242, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void mWeightTxtKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_mWeightTxtKeyReleased

        //get the input string
        String str = mWeightTxt.getText();
        
        //set this error label to be invisible
        mWeightErrorLbl.setVisible(false);    
        mMainScreen.setLaunchButtonEnabled(true);
        
        //try to parse this and set the weight
        try
        {
            float weight = Float.valueOf(str);
            mSelectedFeature.setWeight(weight);
            
        }
        catch(NumberFormatException e)
        {
            System.out.println("error in number");
            mWeightErrorLbl.setVisible(true);
            mMainScreen.setLaunchButtonEnabled(false);
        }
        
    }//GEN-LAST:event_mWeightTxtKeyReleased

    //callback for when we type a key    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JPanel mDetailsWrapper;
    private javax.swing.JTextPane mFeatureDescTxt;
    private com.meapsoft.newgui.checkboxlist.CheckBoxList mFeatureExtractorList;
    private javax.swing.JLabel mFeatureNameLbl;
    private javax.swing.JLabel mWeightErrorLbl;
    private javax.swing.JTextField mWeightTxt;
    // End of variables declaration//GEN-END:variables
    
}
