/*
 *  Copyright 2006-2007 Columbia University.
 *
 *  This file is part of MEAPsoft.
 *
 *  MEAPsoft is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  MEAPsoft is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MEAPsoft; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 *  02110-1301 USA
 *
 *  See the file "COPYING" for the text of the license.
 */

package com.meapsoft.featextractors;

import com.meapsoft.featextractors.AvgMelSpec;

import java.util.Arrays;

import com.meapsoft.DSP;
import com.meapsoft.STFT;

/**
 * Computes the entropy of each chunk.
 * 
 * @author Victor Adan (vga2102@columbia.edu)
 */

public class Entropy extends FeatureExtractor
{

	private AvgMelSpec melSpec;

	static final int swin = 1024;

	static final int nmel = 40;

	static final int sr = 32000;

	public double[] features(STFT stft, long startFrame, int length)
	{

		double[][] frames = new double[length][nmel];
		double[] currFrame;
		double[] workFrame;
		double dist;
		double[][] distMat = new double[length][length];
		double V;
		double[] px = new double[length];
		double[] entropy = new double[1];
		entropy[0] = 0;
		int k = (int) Math.round(Math.sqrt(length));

		melSpec = new AvgMelSpec(swin / 2 + 1, sr, nmel);

		System.out.println("chunck length: " + length);

		// normalization factor
		double maxVal = 0;
		for (int frame = 0; frame < length; frame++)
		{
			frames[frame] = melSpec.features(stft, startFrame + frame, 1);
			currFrame = frames[frame];
			// normalization factor currFrame
			if (DSP.max(currFrame) > maxVal)
			{
				maxVal = DSP.max(currFrame);
			}
		}
		for (int frame = 0; frame < length; frame++)
		{
			currFrame = frames[frame];
			for (int w_frame = frame; w_frame < length; w_frame++)
			{
				workFrame = frames[w_frame];
				dist = 0;
				// get distances
				for (int i = 0; i < nmel; i++)
				{
					dist += Math.pow((currFrame[i] / maxVal - workFrame[i]
							/ maxVal), 2);
				}
				dist = Math.sqrt(dist);
				distMat[frame][w_frame] = dist;
				distMat[w_frame][frame] = dist;
			}
			Arrays.sort(distMat[frame]);

			if (distMat[frame][k] < 0.001)
			{
				System.out.println("Found distance < 0.001");
				distMat[frame][k] = 0.001;
			}
			V = Math.pow(distMat[frame][k], 5);
			px[frame] = ((float) k / length) / V;
		}

		// normalize pmf // this should not be done.
		// double pxsum;
		// pxsum = DSP.sum(px);
		for (int i = 0; i < length; i++)
		{
			// px[i] /= pxsum;
			entropy[0] += px[i] * Math.log(1.0 / px[i]) / Math.log(2);

		}
		System.out.println(entropy[0]);
		return entropy;
	}

	public String description()
	{
		return "Computes the entropy of each chunk.";
	}
}
