/*
 * MeapGUI.java
 *
 * Created on October 27, 2007, 3:31 PM
 */

package com.meapsoft.newgui;

import javax.swing.BoundedRangeModel;
import javax.swing.UIManager;

import com.meapsoft.MEAPUtil;

/**
 *
 * @author  Mike
 */
public class MeapGUI extends javax.swing.JFrame implements Runnable
{
    //our thread
    private Thread mRunThread = null;

    //our tabs
    private SegmenterPanel mSegmenterPanel;
    private ExtractPanel mExtractPanel;
    private ComposePanel mComposePanel;

    //IO data
    public static String mDataDirectory;
    public static String mMeapsoftDirectory;
    
    
    /** Creates new form MeapGUI */
    public MeapGUI()
    {
        initComponents();
        
        //initialize the gui
        initGui();
    }
    
////////////////////////////////////////////////////////////////////////
//getters and setters
////////////////////////////////////////////////////////////////////////
    
    public SegmenterPanel getSegmenterPanel()
    {
    	return mSegmenterPanel;
    }
    
    public ExtractPanel getExtractPanel()
    {
    	return mExtractPanel;
    }
    
    public ComposePanel getComposePanel()
    {
    	return mComposePanel;
    }
    
    public VisualPreviewPanel getPreviewPanel()
    {
        return mPreviewPanel;
    }
    
    public boolean showVisualPreview()
    {
    	return mVisualPreviewChk.isSelected();
    }
    
    public void setLaunchButtonEnabled(boolean flag)
    {
    	mLaunchBtn.setEnabled(flag);	
    }
    
////////////////////////////////////////////////////////////////////////
// updateProgressBar() - does what it says
////////////////////////////////////////////////////////////////////////
    public void updateProgressBar(String txt, BoundedRangeModel model)
    {
    	mProgressLbl.setText(txt);
    	mProgressBar.setStringPainted(true);
    	mProgressBar.setModel(model);
    }
    
////////////////////////////////////////////////////////////////////////
// initGui() - initializes the gui
////////////////////////////////////////////////////////////////////////
    private void initGui()
    {
    	//set the title
    	setTitle("Meapsoft - "+ MEAPUtil.version);
    	
        //set the path here
		String paths[] = MEAPUtil.getPaths();
		if (paths != null)
		{
			mMeapsoftDirectory = paths[0];
			mDataDirectory = paths[1];
		}
		else
			System.exit(-1);
		
        //make the save button disabled for now
        mSaveBtn.setEnabled(false);
        
        //now create all our tabs
        mSegmenterPanel = new SegmenterPanel(this);
        mExtractPanel = new ExtractPanel(this);
        mComposePanel = new ComposePanel(this);
        
        mTabBase.add(mSegmenterPanel, "Segment");
        mTabBase.add(mExtractPanel, "Extract Features");
        mTabBase.add(mComposePanel, "Compose");
        
        //init preview panel
        mPreviewPanel.setVisible(false);
        mPreviewPanel.setMainScreen(this);
    }
    
////////////////////////////////////////////////////////////////////////
 // run() - starts this thread
 ////////////////////////////////////////////////////////////////////////
	public void run() 
	{
        //our error variable
        int error = 0;
        
        //see what we need to do here
        if(mSegmentChkBox.isSelected() && error == 0)
        {
        	error += mSegmenterPanel.run();
        }
        if(mExtractChkBox.isSelected() && error == 0)
        {
        	//error += mSegmenterPanel.run();
        }        
        if(mComposeChkBox.isSelected() && error == 0)
        {
        	//error += mSegmenterPanel.run();
        }
        
        //if we don't have an error here, enable this button
        if(error == 0)
        {
        	mLaunchBtn.setEnabled(true);
        }
        	
	}
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        mTabBase = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        mSegmentChkBox = new javax.swing.JCheckBox();
        mExtractChkBox = new javax.swing.JCheckBox();
        mComposeChkBox = new javax.swing.JCheckBox();
        mLaunchBtn = new javax.swing.JButton();
        mSaveBtn = new javax.swing.JButton();
        mProgressLbl = new javax.swing.JLabel();
        mProgressBar = new javax.swing.JProgressBar();
        mVisualPreviewChk = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        mPreviewPanel = new com.meapsoft.newgui.VisualPreviewPanel();
        mMenuBar = new javax.swing.JMenuBar();
        mFileMenu = new javax.swing.JMenu();
        mAbout = new javax.swing.JMenuItem();
        mPreferences = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        mExit = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Launch", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 0, 11), new java.awt.Color(255, 0, 0)));
        mSegmentChkBox.setSelected(true);
        mSegmentChkBox.setText("Segment");
        mSegmentChkBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        mSegmentChkBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

        mExtractChkBox.setSelected(true);
        mExtractChkBox.setText("Extract Features");
        mExtractChkBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        mExtractChkBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

        mComposeChkBox.setSelected(true);
        mComposeChkBox.setText("Compose");
        mComposeChkBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        mComposeChkBox.setMargin(new java.awt.Insets(0, 0, 0, 0));

        mLaunchBtn.setText("Launch");
        mLaunchBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mLaunchBtnActionPerformed(evt);
            }
        });

        mSaveBtn.setText("Save");

        mProgressLbl.setText("Task Progress");

        mVisualPreviewChk.setSelected(true);
        mVisualPreviewChk.setText("Visual Preview");
        mVisualPreviewChk.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        mVisualPreviewChk.setMargin(new java.awt.Insets(0, 0, 0, 0));

        jLabel1.setForeground(new java.awt.Color(51, 51, 255));
        jLabel1.setText("Options:");

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(mComposeChkBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 105, Short.MAX_VALUE)
                    .add(mExtractChkBox)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(mSegmentChkBox)
                        .addContainerGap(46, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(mProgressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 97, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(mProgressLbl)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(mLaunchBtn, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 95, Short.MAX_VALUE)
                        .addContainerGap())
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(mSaveBtn, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 95, Short.MAX_VALUE)
                        .addContainerGap())))
            .add(jPanel1Layout.createSequentialGroup()
                .add(jLabel1)
                .addContainerGap(74, Short.MAX_VALUE))
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(mVisualPreviewChk)
                .addContainerGap(20, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(mSegmentChkBox)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mExtractChkBox)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mComposeChkBox)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mLaunchBtn)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mSaveBtn)
                .add(50, 50, 50)
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mVisualPreviewChk)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 136, Short.MAX_VALUE)
                .add(mProgressLbl)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mProgressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );

        mFileMenu.setText("File");
        mAbout.setText("About");
        mFileMenu.add(mAbout);

        mPreferences.setText("Preferences");
        mFileMenu.add(mPreferences);

        mFileMenu.add(jSeparator1);

        mExit.setText("Exit");
        mExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mExitActionPerformed(evt);
            }
        });

        mFileMenu.add(mExit);

        mMenuBar.add(mFileMenu);

        setJMenuBar(mMenuBar);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(mTabBase, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 532, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(mPreviewPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 669, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(mTabBase, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 416, Short.MAX_VALUE)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(mPreviewPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void mExitActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_mExitActionPerformed
    {//GEN-HEADEREND:event_mExitActionPerformed
        System.exit(0);
    }//GEN-LAST:event_mExitActionPerformed

    private void mLaunchBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_mLaunchBtnActionPerformed
    {//GEN-HEADEREND:event_mLaunchBtnActionPerformed
//before we start this thread, disable the button
        mLaunchBtn.setEnabled(false);
        
        //create the thread and start it up
        mRunThread = new Thread(this, "MeapGUI.run");
        mRunThread.start();
    }//GEN-LAST:event_mLaunchBtnActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
    	//check to see if we are windows or now.
    	if(System.getProperty("os.name").startsWith("Windows"))
    	{
    		System.out.println("Running windows, putting a nicer look and feel...");
    			
	        try
	        {
	            UIManager.setLookAndFeel("net.java.plaf.windows.WindowsLookAndFeel");
	        }
	        catch (Exception ex)
	        {
	            ex.printStackTrace();
	        }
    	}

        
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                new MeapGUI().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JMenuItem mAbout;
    private javax.swing.JCheckBox mComposeChkBox;
    private javax.swing.JMenuItem mExit;
    private javax.swing.JCheckBox mExtractChkBox;
    private javax.swing.JMenu mFileMenu;
    private javax.swing.JButton mLaunchBtn;
    private javax.swing.JMenuBar mMenuBar;
    private javax.swing.JMenuItem mPreferences;
    private com.meapsoft.newgui.VisualPreviewPanel mPreviewPanel;
    private javax.swing.JProgressBar mProgressBar;
    private javax.swing.JLabel mProgressLbl;
    private javax.swing.JButton mSaveBtn;
    private javax.swing.JCheckBox mSegmentChkBox;
    private javax.swing.JTabbedPane mTabBase;
    private javax.swing.JCheckBox mVisualPreviewChk;
    // End of variables declaration//GEN-END:variables
    
}
