/*
 * VisualPreviewPanel.java
 *
 * Created on October 28, 2007, 2:46 PM
 */

package com.meapsoft.newgui;

import java.awt.Dimension;
import java.util.HashMap;

import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.swing.BoxLayout;
import javax.swing.JPanel;

import com.meapsoft.EDLFile;
import com.meapsoft.FeatFile;
import com.meapsoft.Synthesizer;
import com.meapsoft.composers.BlipComposer;
import com.meapsoft.visualizer.SingleFeaturePanel;
import com.meapsoft.visualizer.SingleFeatureSpectrumPanel;
import com.meapsoft.visualizer.SingleFeatureWaveformPanel;

/**
 *
 * @author  Mike
 */
public class VisualPreviewPanel extends BasePanel implements LineListener
{
    //a hashmap of visual panels
    private HashMap mVisualPanels;
    
    //our playthread
    private Thread playThread = null;
    
    //for previews...
    Synthesizer synth = null; 
    
    // output buffer
    double[] outSamples;
    int outSamplesLength;
	
	
    /** Creates new form VisualPreviewPanel */
    public VisualPreviewPanel()
    {
    	//call our parent with nothing here...just for consistency
    	super(null);
    	
        initComponents();
        
        //create our visual panels
        initVisualPanels();
    }
    
////////////////////////////////////////////////////////////////////////
// setters and getters
////////////////////////////////////////////////////////////////////////
     
     public void setMainScreen(MeapGUI mainScreen)
     {
     	mMainScreen = mainScreen;
     }
     
     public SingleFeaturePanel getVisualPanel(String key)
     {
     	if(mVisualPanels.containsKey(key))
     	{
     		return (SingleFeaturePanel)mVisualPanels.get(key);
     	}
     	
     	return null;
     }
     
////////////////////////////////////////////////////////////////////////
// initVisualPanels() - initializes the visual panels
////////////////////////////////////////////////////////////////////////
        private void initVisualPanels()
        {
        	//create our hashmap first
        	mVisualPanels = new HashMap();
        	
            //instantiate these for use later on
            SingleFeaturePanel waveformPanel = new SingleFeatureWaveformPanel();
            waveformPanel.setMinimumSize(new Dimension(400, 45));
            waveformPanel.setPreferredSize(new Dimension(400,45));

            SingleFeatureSpectrumPanel spectrumPanel = new SingleFeatureSpectrumPanel();
            spectrumPanel.setMinimumSize(new Dimension(400, 45));
            spectrumPanel.setPreferredSize(new Dimension(400,45));	
            
            //add these to our hashmap
            mVisualPanels.put("Waveform", waveformPanel);
            mVisualPanels.put("Spectrum", spectrumPanel);
        }
    
////////////////////////////////////////////////////////////////////////
// addVisualPreview() - adds a visual preview panel
////////////////////////////////////////////////////////////////////////
        
         public void addVisualPreview(String tabName, String featureName, FeatFile featureFile)
         {
             //make sure we are visible in the first place
             this.setVisible(true);
             
             //get the tab with this name and remove its contents
             JPanel previewTab = getPreviewTab(tabName);
              	
             //get the single feature panel here
             SingleFeaturePanel featurePanel = this.getVisualPanel(featureName);

             //update the progress bar here
             String updateString = "Drawing " + featureName + "...";
     	    mMainScreen.updateProgressBar(updateString, featurePanel.getProgress());
     	    
     	    //initialize the panel here
     	    featurePanel.initialize(segmentFile, featureName);
     	    featurePanel.setSegTickType(SingleFeaturePanel.SHORT_SEG_TICKS);
     	    featurePanel.updateData();
             
         	//set visual panel location and add it to the tab
     	    featurePanel.setLocation(10,10);
     	    previewTab.add(featurePanel);
             
     	    //repaint all the things in this tab
     	    previewTab.invalidate();
     	    previewTab.repaint();
     	    
             //repaint and invalidate the tab base
             mVisualTabBase.repaint();
             mVisualTabBase.invalidate();
         } 
         
////////////////////////////////////////////////////////////////////////
// getPreviewTab() - gets a preview tab with this name
////////////////////////////////////////////////////////////////////////
            
        private JPanel getPreviewTab(String tabName)
        {
        	//if we don't have a tab
        	int tabIndex = mVisualTabBase.indexOfTab(tabName);
        	
        	//create a place holder here
        	JPanel tab = null;
        	
        	//if we don't have tab yet, create one here
        	if(tabIndex == -1)
        	{
                //create a tab here and init its layout
            	tab = new JPanel();
            	tab.setLayout(new BoxLayout(tab, BoxLayout.Y_AXIS));
            	
            	//add it as a tab of the tab base
            	mVisualTabBase.add(tab, tabName);
        	}
        	else
        	{
        		tab = (JPanel)mVisualTabBase.getComponent(tabIndex);
        	}
        	
        	//return this tab
        	return tab;
        }
        
    	public synchronized int run() 
    	{
    		return 0;
    	}  
            
    	public void update(LineEvent evt) 
    	{
    		//change the button based on the event here
    		if(evt.getType().equals(LineEvent.Type.START))
    		{
    	        //change the button here
    	        mPreviewBtn.setText("Stop Preview");
    	        mPreviewBtn.setActionCommand("stop");	
    		}
    		else if(evt.getType().equals(LineEvent.Type.STOP))
    		{
        		mPreviewBtn.setText("Listen to Preview");
        		mPreviewBtn.setActionCommand("play");
    		}
    		
    	} 
    	            
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jLabel2 = new javax.swing.JLabel();
        mVisualTabBase = new javax.swing.JTabbedPane();
        mPreviewBtn = new javax.swing.JButton();
        mBlipsChk = new javax.swing.JCheckBox();

        setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Visual Preview", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 0, 11), new java.awt.Color(255, 0, 0)));
        jLabel2.setText("Double click to launch visualizer for more details");

        mPreviewBtn.setText("Listen to Preview");
        mPreviewBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mPreviewBtnActionPerformed(evt);
            }
        });

        mBlipsChk.setSelected(true);
        mBlipsChk.setText("With Blips");
        mBlipsChk.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        mBlipsChk.setMargin(new java.awt.Insets(0, 0, 0, 0));

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 222, Short.MAX_VALUE)
                        .add(mPreviewBtn)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(mBlipsChk))
                    .add(mVisualTabBase, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 643, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(mBlipsChk)
                        .add(mPreviewBtn)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(mVisualTabBase, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 168, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void mPreviewBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_mPreviewBtnActionPerformed
    {//GEN-HEADEREND:event_mPreviewBtnActionPerformed
//see what our action command was here!
        if(evt.getActionCommand().equals("stop"))
        {
            //stop the synthisizer if we were checked
            if(mBlipsChk.isSelected())
            {
                synth.stop();
            }
            else
            {
                
            }
        }
        else
        {
            //if we need to compose with blips, do it here
            if(mBlipsChk.isSelected())
            {
                //create a blip composer and synth it
                EDLFile edl = new EDLFile("null");
                BlipComposer b = new BlipComposer(segmentFile, edl);
                b.setBlipWav(dataDirectory + slash + "blip.wav");
                b.compose();
                
                //create a synth out from this edl file
                synth = new Synthesizer(edl, null);
                
                //get the waveform and add as line listener
                SingleFeatureWaveformPanel wave = (SingleFeatureWaveformPanel)getVisualPanel("Waveform");
                synth.addLineListener(wave);
                synth.addLineListener(this);
                
                //if we have a thread, stop it. LEGACY CODE. unsafe
                if(playThread != null)
                {
                    playThread.interrupt();
                    playThread.stop();
                }
                
                //start this guy up
                playThread = new Thread(synth, "synthesizer");
                playThread.start();
            }
            else
            {
                //add this point we need to use the synth to play
                //without blips. i don't want to redo all that code,
                //so we need to figure out a way to "synth" without
                //actually synthing...
            }
        }
    }//GEN-LAST:event_mPreviewBtnActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel2;
    private javax.swing.JCheckBox mBlipsChk;
    private javax.swing.JButton mPreviewBtn;
    private javax.swing.JTabbedPane mVisualTabBase;
    // End of variables declaration//GEN-END:variables
    
}
