/*
 * CheckBoxList.java
 *
 * Created on October 5, 2007, 7:08 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.meapsoft.newgui.checkboxlist;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionListener;

/**
 * CheckBoxList - a widget that has checkboxes nested inside a JList
 * @author Mike
 */
public class CheckBoxList extends JScrollPane implements MouseListener
{
    private CheckBoxListModel<JCheckBox> mModel;
    private JList mList;
    private int mPreviousSelection;

    /** Creates a new instance of CheckBoxList */
    public CheckBoxList()
    {
        //create the list and the scroll stuff
        mList = new JList();
        mList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        this.setViewportView(mList);
    
        //create the model
        mModel = new CheckBoxListModel<JCheckBox>();
        
        //set all the layout stuff here
        mList.setModel(mModel);
        mList.setCellRenderer(new CheckBoxListRenderer());
        
        //add us as a mouse listener of the list
        mList.addMouseListener(this);
        
        //initialize the previous selection
        mPreviousSelection = -1;
    }

    public void add(JCheckBox kBox)
    {
        mModel.add(kBox);
    }
    
    public void remove(JCheckBox kBox)
    {
        mModel.remove(kBox);
    }
    
    public Vector getSelected()
    {
        //create the vector here
        Vector kVec = new Vector();
        
        //iterate through all the checkboxes
        for(int i = 0; i < mModel.getSize(); i++)
        {
            JCheckBox kBox = (JCheckBox)mModel.getElementAt(i);
            
            //if that box is selected, add it to the vector
            if(kBox.isSelected())
            {
                kVec.add(kBox.getText());
            }
        }
        
        //return this vector
        return kVec;
    }    
    
    //return the selected item in our list
    public JCheckBox getSelectedValue()
    {
    	return (JCheckBox)mList.getSelectedValue();
    }
    
    //just a facade for our listbox
    public void addListSelectionLister(ListSelectionListener listener)
    {
    	mList.addListSelectionListener(listener);
    }
    
    public void removeListSelectionLister(ListSelectionListener listener)
    {
    	mList.addListSelectionListener(listener);
    }

    public void mouseClicked(MouseEvent e)
    {
    }

    public void mousePressed(MouseEvent e)
    {
    }

    public void mouseReleased(MouseEvent e)
    {
        //get the index of the selected item
        int iSelected = mList.getSelectedIndex();
        
        //get the checkbox
        JCheckBox kBox = (JCheckBox)mModel.getElementAt(iSelected);
        
        //if it wasnt the previously selected one, just return
        if(iSelected != mPreviousSelection)
        {
            mPreviousSelection = iSelected;
            return;
        }
        
        //if not, set it here either way
         mPreviousSelection = iSelected;
        
        //update the check itself
        kBox.setSelected(!kBox.isSelected());
        
        //repaint and invalidate the box
        this.invalidate();
        this.repaint();
    }

    public void mouseEntered(MouseEvent e)
    {
    }

    public void mouseExited(MouseEvent e)
    {
    }
}
